import logging
import traceback
from time import time
from reportportal_client.service import (
    _dict_to_payload,
    ReportPortalService
)

from .variables import Variables


def async_error_handler(exc_info):
    exc, msg, tb = exc_info
    traceback.print_exception(exc, msg, tb)


def timestamp():
    return str(int(time() * 1000))


class RobotService(object):
    rp = None

    status_mapping = {
        "PASS": "PASSED",
        "FAIL": "FAILED",
        "SKIP": "SKIPPED"
    }

    log_level_mapping = {
        "INFO": "INFO",
        "FAIL": "ERROR",
        "TRACE": "TRACE",
        "DEBUG": "DEBUG",
        "HTML": "INFO",
        "WARN": "WARN",
        "ERROR": "ERROR"
    }

    @staticmethod
    def _get_launch_attributes(cmd_attrs):
        """Generate launch attributes including both system and user ones.

        :param list cmd_attrs: List for attributes from the pytest.ini file
        """
        attributes = cmd_attrs or []
        system_info = RobotService.rp.get_system_information(
            Variables.agent_name)
        system_info['system'] = True
        system_attributes = _dict_to_payload(system_info)
        return attributes + system_attributes

    @staticmethod
    def init_service(endpoint, project, uuid):
        if RobotService.rp is None:
            logging.debug(
                "ReportPortal - Init service: "
                "endpoint={0}, project={1}, uuid={2}"
                .format(endpoint, project, uuid))
            RobotService.rp = ReportPortalService(
                endpoint=endpoint,
                project=project,
                token=uuid)
        else:
            raise Exception("RobotFrameworkService is already initialized")

    @staticmethod
    def terminate_service():
        if RobotService.rp is not None:
            RobotService.rp.terminate()

    @staticmethod
    def start_launch(launch_name, start_time, attributes=None, description=None, mode=None):
        """Call start_launch method of the common client.

        :param launch_name: Launch name
        :param attributes:  Launch attributes
        :param description: Launch description
        :param mode:        Launch mode
        :return:            launch UUID
        """
        sl_pt = {
            "attributes": RobotService._get_launch_attributes(attributes),
            "name": launch_name,
            "start_time": start_time,
            "description": description,
            "mode": mode
        }
        logging.debug("ReportPortal - Start launch: "
                      "request_body={0}".format(sl_pt))
        return RobotService.rp.start_launch(**sl_pt)

    @staticmethod
    def finish_launch(launch, end_time):
        fl_rq = {
            "end_time": end_time,
            "status": RobotService.status_mapping[launch.status]
        }
        logging.debug("ReportPortal - Finish launch: "
                      "request_body={0}".format(fl_rq))
        RobotService.rp.finish_launch(**fl_rq)

    @staticmethod
    def start_suite(name, start_time, suite=None, parent_item_id=None, attributes=None):
        start_rq = {
            "name": name,
            "attributes": attributes,
            "description": suite.doc,
            "start_time": start_time,
            "item_type": "SUITE",
            "parent_item_id": parent_item_id
        }
        logging.debug(
            "ReportPortal - Start suite: "
            "request_body={0}".format(start_rq))
        return RobotService.rp.start_test_item(**start_rq)

    @staticmethod
    def finish_suite(item_id, end_time, issue=None, suite=None):
        fta_rq = {
            "end_time": end_time,
            "status": RobotService.status_mapping[suite.status],
            "issue": issue,
            "item_id": item_id
        }
        logging.debug(
            "ReportPortal - Finish suite:"
            " request_body={0}".format(fta_rq))
        RobotService.rp.finish_test_item(**fta_rq)

    @staticmethod
    def start_test(test, start_time, parent_item_id=None, attributes=None):
        # Item type should be sent as "STEP" until we upgrade to RPv6.
        # Details at: https://github.com/reportportal/agent-Python-RobotFramework/issues/56
        start_rq = {
            "name": test.name,
            "attributes": attributes,
            "description": test.doc,
            "start_time": start_time,
            "item_type": "STEP",
            "parent_item_id": parent_item_id
        }
        logging.debug(
            "ReportPortal - Start test: "
            "request_body={0}".format(start_rq))
        return RobotService.rp.start_test_item(**start_rq)

    @staticmethod
    def finish_test(item_id, end_time, issue=None, test=None):
        fta_rq = {
            "end_time": end_time,
            "status": RobotService.status_mapping[test.status],
            "issue": issue,
            "item_id": item_id
        }
        logging.debug(
            "ReportPortal - Finish test:"
            " request_body={0}".format(fta_rq))
        RobotService.rp.finish_test_item(**fta_rq)

    @staticmethod
    def start_keyword(keyword, start_time, parent_item_id=None, has_stats=True):
        start_rq = {
            "name": keyword.get_name(),
            "description": keyword.doc,
            "start_time": start_time,
            "item_type": keyword.get_type(),
            "parent_item_id": parent_item_id,
            "has_stats": has_stats
        }
        logging.debug(
            "ReportPortal - Start keyword: "
            "request_body={0}".format(start_rq))
        return RobotService.rp.start_test_item(**start_rq)

    @staticmethod
    def finish_keyword(item_id, end_time, issue=None, keyword=None):
        fta_rq = {
            "end_time": end_time,
            "status": RobotService.status_mapping[keyword.status],
            "issue": issue,
            "item_id": item_id
        }
        logging.debug(
            "ReportPortal - Finish keyword:"
            " request_body={0}".format(fta_rq))
        RobotService.rp.finish_test_item(**fta_rq)

    @staticmethod
    def log(message, log_time):
        sl_rq = {
            "time": log_time,
            "message": message.message,
            "level": RobotService.log_level_mapping[message.level],
            "attachment": message.attachment,
            "item_id": message.item_id
        }
        RobotService.rp.log(**sl_rq)



import pytest
import io
from rpbot.reader.robot_results_parser import RobotResultsParser


@pytest.fixture
def reporter(mocker):
    reporter = mocker.MagicMock()
    yield reporter


@pytest.fixture
def parser(reporter):
    parser = RobotResultsParser(reporter)
    yield parser


simple_output_xml = """<?xml version="1.0" encoding="UTF-8"?>
<robot generator="Robot 3.2.1 (Python 3.8.2 on linux)" generated="20200926 15:33:43.760" rpa="false">
<suite id="s1" name="Test Rp" source="/p4_ws/doyou89.jung/workspace/projects/cosmos/atest/example/test_rp.robot">
<test id="s1-t1" name="test1">
<kw name="Log" library="BuiltIn">
<doc>Logs the given message with the given level.</doc>
<arguments>
<arg>test1</arg>
</arguments>
<msg timestamp="20200926 15:33:43.777" level="INFO">test1</msg>
<status status="PASS" starttime="20200926 15:33:43.777" endtime="20200926 15:33:43.777"></status>
</kw>
<status status="PASS" starttime="20200926 15:33:43.776" endtime="20200926 15:33:43.777" critical="yes"></status>
</test>
<status status="PASS" starttime="20200926 15:33:43.761" endtime="20200926 15:33:43.777"></status>
</suite>
<statistics>
<total>
<stat pass="1" fail="0">Critical Tests</stat>
<stat pass="1" fail="0">All Tests</stat>
</total>
<tag>
</tag>
<suite>
<stat pass="1" fail="0" id="s1" name="Test Rp">Test Rp</stat>
</suite>
</statistics>
<errors>
</errors>
</robot>
"""

simple_failed_output_xml = """<?xml version="1.0" encoding="UTF-8"?>
<robot generator="Robot 3.2.1 (Python 3.8.2 on linux)" generated="20200926 15:34:56.754" rpa="false">
<suite id="s1" name="Test Rp" source="/p4_ws/doyou89.jung/workspace/projects/cosmos/atest/example/test_rp.robot">
<test id="s1-t1" name="test1">
<kw name="Log" library="BuiltIn">
<doc>Logs the given message with the given level.</doc>
<arguments>
<arg>test1</arg>
</arguments>
<msg timestamp="20200926 15:34:56.771" level="INFO">test1</msg>
<status status="PASS" starttime="20200926 15:34:56.771" endtime="20200926 15:34:56.771"></status>
</kw>
<kw name="Fail" library="BuiltIn">
<doc>Fails the test with the given message and optionally alters its tags.</doc>
<arguments>
<arg>test fail message</arg>
</arguments>
<msg timestamp="20200926 15:34:56.771" level="FAIL">test fail message</msg>
<status status="FAIL" starttime="20200926 15:34:56.771" endtime="20200926 15:34:56.771"></status>
</kw>
<status status="FAIL" starttime="20200926 15:34:56.770" endtime="20200926 15:34:56.771" critical="yes">test fail message</status>
</test>
<status status="FAIL" starttime="20200926 15:34:56.755" endtime="20200926 15:34:56.772"></status>
</suite>
<statistics>
<total>
<stat pass="0" fail="1">Critical Tests</stat>
<stat pass="0" fail="1">All Tests</stat>
</total>
<tag>
</tag>
<suite>
<stat pass="0" fail="1" id="s1" name="Test Rp">Test Rp</stat>
</suite>
</statistics>
<errors>
</errors>
</robot>
"""


def test_xml_to_db(reporter, parser):
    """test xml_to_db"""
    parser.xml_to_db(io.StringIO(simple_output_xml))

    assert reporter.start_suite.call_count == 1
    assert reporter.start_suite.call_args[0][0] == 'Test Rp'
    assert reporter.start_suite.call_args[0][1]['id'] == 's1'
    assert reporter.start_suite.call_args[0][1]['status'] == 'PASS'

    assert reporter.start_test.call_count == 1
    assert reporter.start_test.call_args[0][0] == 'test1'
    assert reporter.start_test.call_args[0][1]['id'] == 's1-t1'
    assert reporter.start_test.call_args[0][1]['status'] == 'PASS'

    assert reporter.start_keyword.call_count == 1
    assert reporter.start_keyword.call_args[0][0] == 'BuiltIn.Log'
    assert reporter.start_keyword.call_args[0][1]['type'] == 'kw'
    assert reporter.start_keyword.call_args[0][1]['kwname'] == 'Log'
    assert reporter.start_keyword.call_args[0][1]['libname'] == 'BuiltIn'
    assert reporter.start_keyword.call_args[0][1]['status'] == 'PASS'

    assert reporter.end_keyword.call_count == 1
    assert reporter.end_test.call_count == 1
    assert reporter.end_suite.call_count == 2


def test_xml_to_db_with_fail(reporter, parser):
    """test xml_to_db with failed test"""
    parser.xml_to_db(io.StringIO(simple_failed_output_xml))

    assert reporter.start_suite.call_count == 1
    assert reporter.start_suite.call_args[0][0] == 'Test Rp'
    assert reporter.start_suite.call_args[0][1]['id'] == 's1'
    assert reporter.start_suite.call_args[0][1]['status'] == 'FAIL'

    assert reporter.start_test.call_count == 1
    assert reporter.start_test.call_args[0][0] == 'test1'
    assert reporter.start_test.call_args[0][1]['id'] == 's1-t1'
    assert reporter.start_test.call_args[0][1]['status'] == 'FAIL'

    assert reporter.start_keyword.call_count == 2
    assert reporter.start_keyword.call_args_list[0][0][0] == 'BuiltIn.Log'
    assert reporter.start_keyword.call_args_list[0][0][1]['type'] == 'kw'
    assert reporter.start_keyword.call_args_list[0][0][1]['kwname'] == 'Log'
    assert reporter.start_keyword.call_args_list[0][0][1]['libname'] == 'BuiltIn'
    assert reporter.start_keyword.call_args_list[0][0][1]['status'] == 'PASS'

    assert reporter.end_keyword.call_count == 2
    assert reporter.end_test.call_count == 1
    assert reporter.end_suite.call_count == 2



#!/usr/bin/env python
#  Copyright 2013-2014 Nokia Solutions and Networks
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
import sys

from robot.errors import DataError
from rpbot.reader import ReaderOptions, RobotResultsParser
from rpbot.reportportal.variables import Variables
from rpbot.reportportal.reporter import DryRunRP, ReportPortal


class RpBot(object):

    def __init__(self):
        self._options = ReaderOptions()
        Variables.check_variables(self._options)
        if self._options.dry_run:
            reporter = DryRunRP(self._options.verbose_level)
        else:
            reporter = ReportPortal(self._options.verbose_level)
        self._parser = RobotResultsParser(reporter, self._options.verbose_level)

    def run(self):
        try:
            for xml_file in self._options.file_paths:
                self._parser.xml_to_db(xml_file)
        except DataError as message:
            sys.stderr.write('rpbot: error: Invalid XML: %s\n\n' % message)
            exit(1)
        # finally:
        #     self._db.close()


if __name__ == '__main__':
    RpBot().run()



#  Copyright 2013-2014 Nokia Solutions and Networks
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
from optparse import OptionParser, OptionGroup
from os.path import exists


class ReaderOptions(object):

    def __init__(self):
        self._parser = OptionParser()
        self._add_parser_options()
        self._options, self._files = self._get_validated_options()

    def _add_parser_options(self):
        options = [
            ('-d', '--dry-run', {'action': 'store_true',
                                 'default': False,
                                 'dest': 'dry_run',
                                 'help': 'do everything except store results into reportportal'}),

            ('-v', '--verbose', {
                                 'default': False,
                                 'dest': 'level',
                                 'help': 'be verbose. WARN, INFO, and DEBUG available.'}),
        ]
        for option in options:
            self._parser.add_option(option[0], option[1], **option[2])

        rp_group_options = [
            ('', '--RP_UUID', {'dest': 'rp_uuid', 'default': None, 'help': 'your user uuid'}),
            ('', '--RP_ENDPOINT', {'dest': 'rp_endpoint', 'default': None, 'help': 'your reportportal url'}),
            ('', '--RP_LAUNCH', {'dest': 'rp_launch', 'default': None, 'help': 'launch name'}),
            ('', '--RP_PROJECT', {'dest': 'rp_project', 'default': None, 'help': 'reportportal project name'}),
            ('', '--RP_LAUNCH_UUID', {'dest': 'rp_launch_uuid', 'default': None,
                                      'help': 'ID of existing reportportal launch'}),
            ('', '--RP_LAUNCH_DOC', {'dest': 'rp_launch_doc', 'default': None,
                                     'help': 'Description  for the launch'}),
            ('', '--RP_LAUNCH_ATTRIBUTES', {'dest': 'rp_launch_attributes', 'default': '',
                                            'help': 'Space-separated list of tags/attributes for the launch'}),
            ('', '--RP_TEST_ATTRIBUTES', {'dest': 'rp_test_attributes', 'default': '',
                                          'help': 'Space-separated list of tags/attributes for the tests'}),
            ('', '--RP_LOG_BATCH_SIZE', {'dest': 'rp_log_batch_size', 'default': '20',
                                         'help': 'Default value is "20", affects size of async batch log requests'}),
        ]
        rp_group = OptionGroup(self._parser, 'ReportPortal Options')
        for option in rp_group_options:
            rp_group.add_option(option[0], option[1], **option[2])
        self._parser.add_option_group(rp_group)

    def _get_validated_options(self):
        options, files = self._parser.parse_args()
        self._check_files(files)
        return options, files

    def _check_files(self, files):
        if not files or len(files) < 1:
            self._parser.error('at least one input file is required')
        for file_path in files:
            if not exists(file_path):
                self._parser.error('file "%s" does not exist' % file_path)

    def _exit_with_help(self):
        self._parser.print_help()
        exit(1)

    @property
    def verbose_level(self):
        return self._options.level

    @property
    def file_paths(self):
        return self._files

    @property
    def dry_run(self):
        return self._options.dry_run

    @property
    def rp_uuid(self):
        return self._options.rp_uuid

    @property
    def rp_endpoint(self):
        return self._options.rp_endpoint

    @property
    def rp_launch(self):
        return self._options.rp_launch

    @property
    def rp_project(self):
        return self._options.rp_project

    @property
    def rp_launch_uuid(self):
        return self._options.rp_launch_uuid

    @property
    def rp_launch_doc(self):
        return self._options.rp_launch_doc

    @property
    def rp_launch_attributes(self):
        return self._options.rp_launch_attributes

    @property
    def rp_test_attributes(self):
        return self._options.rp_test_attributes

    @property
    def rp_log_batch_size(self):
        return self._options.rp_log_batch_size



from six import text_type

from reportportal_client.service import _convert_string


class Suite(object):
    def __init__(self, attributes):
        super(Suite, self).__init__()
        self.attributes = attributes
        self.suites = attributes["suites"]
        self.tests = attributes["tests"]
        self.doc = attributes["doc"]
        self.source = attributes["source"]
        self.total_tests = attributes["totaltests"]
        self.longname = attributes["longname"]
        self.robot_id = attributes["id"]
        self.metadata = attributes["metadata"]
        self.start_time = attributes["starttime"]
        self.end_time = attributes["endtime"]
        self.status = None
        self.message = None
        self.statistics = None
        if "status" in attributes.keys():
            self.status = attributes["status"]
        if "message" in attributes.keys():
            self.message = attributes["message"]
        if "statistics" in attributes.keys():
            self.statistics = attributes["statistics"]


class Test(object):
    def __init__(self, name=None, attributes=None):
        super(Test, self).__init__()
        self.name = name
        self.critical = attributes["critical"]
        self.template = attributes["template"]
        self.tags = attributes["tags"]
        self.doc = attributes["doc"]
        self.longname = attributes["longname"]
        self.robot_id = attributes["id"]
        self.start_time = attributes["starttime"]
        self.end_time = attributes["endtime"]
        self.status = None
        self.message = None
        if "status" in attributes.keys():
            self.status = attributes["status"]
        if "message" in attributes.keys():
            self.message = attributes["message"]


class Keyword(object):
    def __init__(self, name=None, parent_type="TEST", attributes=None):
        super(Keyword, self).__init__()
        self.name = name
        self.libname = attributes["libname"]
        self.keyword_name = attributes["kwname"]
        self.doc = attributes["doc"]
        self.tags = attributes["tags"]
        self.args = attributes["args"]
        self.assign = attributes["assign"]
        self.keyword_type = attributes["type"]
        self.parent_type = parent_type
        self.start_time = attributes["starttime"]
        self.end_time = attributes["endtime"]
        if "status" in attributes.keys():
            self.status = attributes["status"]

    def get_name(self):
        assign = _convert_string(", ".join(self.assign))
        assignment = "{0} = ".format(assign) if self.assign else ""
        arguments = ", ".join(self.args)
        full_name = "{0}{1} ({2})".format(
            assignment,
            _convert_string(self.name),
            _convert_string(arguments)
        )
        return full_name[:256]

    def get_type(self):
        if self.keyword_type == "Setup":
            return "BEFORE_{0}".format(self.parent_type)
        elif self.keyword_type == "Teardown":
            return "AFTER_{0}".format(self.parent_type)
        else:
            return "STEP"


class LogMessage(text_type):
    def __init__(self, *args, **kwargs):
        super(LogMessage, self).__init__()
        self.item_id = None
        self.message = self
        self.level = "INFO"
        self.attachment = None



class Variables(object):
    agent_name = 'robotframework-reportportal'
    uuid = None
    endpoint = None
    launch_name = None
    project = None
    launch_doc = None
    log_batch_size = None
    launch_attributes = None
    launch_id = None
    test_attributes = None

    @staticmethod
    def check_variables(options):
        Variables.uuid = options.rp_uuid
        Variables.endpoint = options.rp_endpoint
        Variables.launch_name = options.rp_launch
        Variables.project = options.rp_project
        Variables.launch_attributes = options.rp_launch_attributes.split()
        Variables.launch_id = options.rp_launch_uuid
        Variables.launch_doc = options.rp_launch_doc
        Variables.log_batch_size = int(options.rp_log_batch_size)
        Variables.test_attributes = options.rp_test_attributes.split()



#  Copyright 2013-2014 Nokia Solutions and Networks
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
import datetime
import time
import logging
from robot.api import ExecutionResult


logging.basicConfig()


class RobotResultsParser(object):

    def __init__(self, reporter, verbose=False):
        self._logger = logging.getLogger('Parser')
        if verbose:
            self._logger.setLevel(verbose)
        self.reporter = reporter
        self.start_time = None
        self.end_time = None

    def xml_to_db(self, xml_file):
        self._logger.info('- Parsing %s' % xml_file)
        test_run = ExecutionResult(xml_file, include_keywords=True)

        self._find_start_end_time(test_run.suite)

        self._parse_suite(test_run.suite)

    def _find_start_end_time(self, suite):
        self.start_time = self._find_start_time(suite)
        self.end_time = self._find_end_time(suite)

    def _find_start_time(self, suite):
        if suite.starttime:
            return suite.starttime
        if suite.suites:
            return self._find_start_time(suite.suites[0])
        if suite.tests:
            return suite.tests[0].starttime
        if suite.keywords:
            return suite.keywords[0].starttime

    def _find_end_time(self, suite):
        if suite.endtime:
            return suite.endtime
        if suite.suites:
            return self._find_end_time(suite.suites[-1])
        if suite.tests:
            return suite.tests[-1].starttime
        if suite.keywords:
            return suite.keywords[-1].starttime

    def _timestamp(self, t_str):
        return int(datetime.datetime.strptime(t_str, '%Y%m%d %H:%M:%S.%f').timestamp() * 1000)

    def _parse_suite(self, suite):
        self._logger.info('`--> Parsing suite: %s' % suite.name)

        attributes = {
            'id': suite.id,
            'longname': suite.longname,
            'doc': suite.doc,
            'metadata': suite.metadata,
            'source': suite.source,
            'suites': suite.suites,
            'tests': suite.tests,
            'totaltests': suite.test_count,
            'starttime': self._timestamp(suite.starttime or self.start_time),
            'endtime': self._timestamp(suite.endtime or self.end_time),
            'status': suite.status,
            'statistics': suite.statistics,
            'message': suite.message,
        }

        self.reporter.start_suite(suite.name, attributes)

        self._parse_suites(suite)
        self._parse_tests(suite.tests)
        self._parse_keywords(suite.keywords)

        self.reporter.end_suite(suite.name, attributes)
        if suite.id == 's1' and not suite.suites:
            attributes['id'] = suite.id
            self.reporter.end_suite(suite.name, attributes)

    def _parse_suites(self, suite):
        [self._parse_suite(subsuite) for subsuite in suite.suites]

    def _parse_tests(self, tests):
        [self._parse_test(test) for test in tests]

    def _parse_test(self, test):
        self._logger.info('  `--> Parsing test: %s' % test.name)

        attributes = {
            'id': test.id,
            'longname': test.longname,
            'doc': test.doc,
            'tags': [tag for tag in test.tags],
            'critical': test.critical,
            'template': '',
            'starttime': self._timestamp(test.starttime),
            'endtime': self._timestamp(test.endtime),
            'status': test.status,
            'message': test.message
        }

        self.reporter.start_test(test.name, attributes)

        self._parse_keywords(test.keywords)

        self.reporter.end_test(test.name, attributes)

    def _parse_keywords(self, keywords):
        [self._parse_keyword(keyword) for keyword in keywords]

    def _parse_keyword(self, keyword):

        attributes = {
            'type': keyword.type,
            'kwname': keyword.kwname,
            'libname': keyword.libname,
            'doc': keyword.doc,
            'args': keyword.args,
            'assign': keyword.assign,
            'tags': keyword.tags,
            'starttime': self._timestamp(keyword.starttime),
            'endtime': self._timestamp(keyword.endtime),
            'status': keyword.status
        }

        self.reporter.start_keyword(keyword.name, attributes)

        self._parse_messages(keyword.messages)
        self._parse_keywords(keyword.keywords)

        self.reporter.end_keyword(keyword.name, attributes)

    def _parse_messages(self, messages):
        for message in messages:
            self.reporter.log_message({'message': message.message, 'level': message.level,
                                       'timestamp': self._timestamp(message.timestamp), 'html': message.html})



#!/usr/bin/env python

from distutils.core import setup
from os.path import abspath, dirname, join
import re
from setuptools import setup, find_packages

NAME = 'rpbot'
CLASSIFIERS = """
Development Status :: 4 - Beta
Operating System :: OS Independent
Programming Language :: Python
Topic :: Software Development :: Testing
""".strip().splitlines()
CURDIR = dirname(abspath(__file__))
with open(join(CURDIR, 'rpbot', '__init__.py')) as f:
    VERSION = re.search("\n__version__ = '(.*)'\n", f.read()).group(1)
with open(join(CURDIR, 'README.md')) as f:
    README = f.read()
CURDIR = dirname(abspath(__file__))
with open(join(CURDIR, 'requirements.txt')) as f:
    REQUIREMENTS = f.read().splitlines()
    REQUIREMENTS = [r.split('#egg=')[-1] for r in REQUIREMENTS]

setup(
    name             = NAME,
    version          = VERSION,
    author           = 'Doyou Jung',
    author_email     = 'doyou89@gmail.com',
    url              = 'https://github.com/doyou89/RPBot',
    download_url     = 'https://pypi.python.org/pypi/rpbot',
    license          = 'Apache License 2.0',
    description      = 'A tool for inserting Robot Framework test run results into ReportPortal.',
    long_description = README,
    long_description_content_type = 'text/markdown',
    keywords         = 'robotframework testing testautomation atdd',
    platforms        = 'any',
    classifiers      = CLASSIFIERS,
    packages         = find_packages(),
    install_requires = REQUIREMENTS,
)



import logging
from reportportal_client.helpers import gen_attributes

from .variables import Variables
from .model import Keyword, Test, Suite, LogMessage
from .service import RobotService


class DryRunRP:

    def __init__(self, verbose=False):
        self._logger = logging.getLogger('RP(DRYRUN)')
        if verbose:
            self._logger.setLevel(verbose)

    def __getattr__(self, name):
        def method(*args):
            self._logger.debug("call to " + name)
        return method

class ReportPortal:

    def __init__(self, verbose=False):
        self.items = []
        self._logger = logging.getLogger('RP')
        if verbose:
            self._logger.setLevel(verbose)

    def _gen_attributes_from_robot_tags(self, tags):
        attrs = []
        for tag in tags:
            attrs.append({'value': tag})
        return attrs

    def start_launch(self, launch):
        """Start a new launch at the Report Portal."""
        if not Variables.launch_id:
            launch.doc = Variables.launch_doc
            self._logger.debug("ReportPortal - Start Launch: {0}".format(
                launch.attributes))
            RobotService.start_launch(
                launch_name=Variables.launch_name,
                start_time=launch.start_time,
                attributes=gen_attributes(Variables.launch_attributes),
                description=launch.doc)
        else:
            RobotService.rp.launch_id = Variables.launch_id

    def start_suite(self, name, attributes):
        suite = Suite(attributes=attributes)
        if suite.robot_id == "s1":
            RobotService.init_service(Variables.endpoint, Variables.project,
                                    Variables.uuid)
            self.start_launch(suite)
            if not suite.suites:
                attributes['id'] = "s1-s1"
                self.start_suite(name, attributes)
        else:
            self._logger.debug("ReportPortal - Start Suite: {0}".format(attributes))
            parent_id = self.items[-1][0] if self.items else None
            item_id = RobotService.start_suite(
                name=name,
                start_time=suite.start_time,
                suite=suite,
                parent_item_id=parent_id)
            self.items.append((item_id, parent_id))

    def end_suite(self, _, attributes):
        suite = Suite(attributes=attributes)
        if suite.robot_id == "s1":
            self._logger.debug(msg="ReportPortal - End Launch: {0}".format(attributes))
            RobotService.finish_launch(launch=suite, end_time=suite.end_time)
            RobotService.terminate_service()
        else:
            self._logger.debug("ReportPortal - End Suite: {0}".format(attributes))
            RobotService.finish_suite(item_id=self.items.pop()[0], end_time=suite.end_time, suite=suite)

    def start_test(self, name, attributes):
        test = Test(name=name, attributes=attributes)
        self._logger.debug("ReportPortal - Start Test: {0}".format(attributes))
        parent_item_id = self.items[-1][0]
        self.items.append((
            RobotService.start_test(
                test=test,
                start_time=test.start_time,
                parent_item_id=parent_item_id,
                attributes=gen_attributes(Variables.test_attributes) + self._gen_attributes_from_robot_tags(test.tags),
            ),
            parent_item_id))

    def end_test(self, name, attributes):
        test = Test(name=name, attributes=attributes)
        item_id, _ = self.items.pop()
        self._logger.debug("ReportPortal - End Test: {0}".format(attributes))
        RobotService.finish_test(item_id=item_id, end_time=test.end_time, test=test)

    def start_keyword(self, name, attributes):
        parent_type = 'SUITE' if not self.items else 'TEST'
        parent_item_id = self.items[-1][0]
        kwd = Keyword(name=name, parent_type=parent_type, attributes=attributes)
        self._logger.debug("ReportPortal - Start Keyword: {0}".format(attributes))
        self.items.append((
            RobotService.start_keyword(keyword=kwd, start_time=kwd.start_time, parent_item_id=parent_item_id,
                                       has_stats=False),
            parent_item_id))

    def end_keyword(self, name, attributes):
        kwd = Keyword(name=name, attributes=attributes)
        item_id, _ = self.items.pop()
        self._logger.debug("ReportPortal - End Keyword: {0}".format(attributes))
        RobotService.finish_keyword(item_id=item_id, end_time=kwd.end_time, keyword=kwd)

    def log_message(self, message):
        # Check if message comes from our custom logger or not
        if isinstance(message["message"], LogMessage):
            msg = message["message"]
        else:
            msg = LogMessage(message["message"])
            msg.level = message["level"]

        msg.item_id = self.items[-1][0]
        self._logger.debug("ReportPortal - Log Message: {0}".format(message))
        RobotService.log(message=msg, log_time=message["timestamp"])

    def message(self, message):
        self._logger.debug("ReportPortal - Message: {0}".format(message))

    def library_import(self, name, attributes):
        self._logger.debug("ReportPortal - Library Import: {0}".format(attributes))

    def resource_import(self, name, attributes):
        self._logger.debug("ReportPortal - Resource Import: {0}".format(attributes))

    def variables_import(self, name, attributes):
        self._logger.debug("ReportPortal - Variables Import: {0}".format(attributes))

    def output_file(self, path):
        self._logger.debug("ReportPortal - Output File: {0}".format(path))

    def log_file(self, path):
        self._logger.debug("ReportPortal - Log File: {0}".format(path))

    def report_file(self, path):
        self._logger.debug("ReportPortal - Report File: {0}".format(path))

    def xunit_file(self, path):
        self._logger.debug("ReportPortal - XUnit File: {0}".format(path))

    def info_file(self, path):
        self._logger.debug("ReportPortal - info File: {0}".format(path))
